import sys
import requests
from colorama import Style, Fore

from howfairis.mixins import RepositoryMixin
from howfairis.mixins import LicenseMixin
from howfairis.mixins import RegistryMixin
from howfairis.mixins import CitationMixin
from howfairis.mixins import ChecklistMixin


class HowFairIsChecker(RepositoryMixin, LicenseMixin, RegistryMixin, CitationMixin, ChecklistMixin):
    def __init__(self, url):
        super().__init__()
        assert url.startswith("https://github.com/"), \
            "url should start with https://github.com"
        self.url = url
        self.readme = None
        self.repository_is_compliant = None
        self.license_is_compliant = None
        self.registry_is_compliant = None
        self.citation_is_compliant = None
        self.checklist_is_compliant = None
        self.badge = None
        self.owner = None
        self.repo = None
        self.readme_filename = None
        self.branch = None
        self.compliant_symbol = "\u25CF"
        self.noncompliant_symbol = "\u25CB"

    def check_badge(self):

        compliance_bool = [
            self.repository_is_compliant, self.license_is_compliant,
            self.registry_is_compliant, self.citation_is_compliant,
            self.checklist_is_compliant
        ]

        compliance_unicode = [None] * 5
        for i, c in enumerate(compliance_bool):
            if c is True:
                compliance_unicode[i] = self.compliant_symbol
            else:
                compliance_unicode[i] = self.noncompliant_symbol

        compliance_string = "%20%20".join(
            [requests.utils.quote(symbol) for symbol in compliance_unicode])

        score = compliance_bool.count(True)
        if score in [0, 1]:
            color_string = "red"
        elif score in [2, 3, 4]:
            color_string = "orange"
        elif score == 5:
            color_string = "green"

        self.badge = "[![fair-software.eu](https://img.shields.io/badge/fair--software.eu-{0}-{1})]({2})"\
                     .format(compliance_string, color_string, "https://fair-software.eu")

        print("\nCalculated compliance: " + " ".join(compliance_unicode) + "\n")

        if self.readme is None:
            sys.exit(1)
        elif self.readme.find(self.badge) == -1:
            print("While searching through your README.md, I" +
                  " did not find the expected badge:\n" + self.badge + "\n")
            sys.exit(1)
        else:
            print("Expected badge is equal to the actual badge. " +
                  "It's all good.\n")
            sys.exit(0)

    def check_checklist(self):
        print("(5/5) checklist")
        results = [
            self.has_core_infrastructures_badge(),
            self.has_sonarcloud_badge()
        ]
        self.checklist_is_compliant = True in results
        return self

    def check_citation(self):
        print("(4/5) citation")
        results = [
            self.has_zenodo_badge(),
            self.has_citationcff_file(),
            self.has_citation_file(),
            self.has_zenodo_metadata_file(),
            self.has_codemeta_file()
        ]
        self.citation_is_compliant = True in results
        return self

    def check_license(self):
        print("(2/5) license")
        results = [self.has_license()]
        self.license_is_compliant = True in results
        return self

    def check_registry(self):
        print("(3/5) registry")
        results = [
            self.has_pypi_badge(),
            self.has_conda_badge(),
            self.has_bintray_badge(),
            self.is_on_github_marketplace()
        ]
        self.registry_is_compliant = True in results
        return self

    def check_repository(self):
        print("(1/5) repository")
        results = [self.has_open_repository()]
        self.repository_is_compliant = True in results
        return self

    def deconstruct_url(self):
        self.owner, self.repo = self.url.replace("https://github.com/",
                                                 "").split("/")[:2]
        self.branch = "master"
        self.readme_filename = "README.md"
        return self

    def get_readme(self):
        # only github urls supported
        # only README.md supported

        raw_url = "https://raw.githubusercontent.com/" + \
                  "{0}/{1}/{2}/{3}".format(self.owner, self.repo,
                                           self.branch, self.readme_filename)
        try:
            response = requests.get(raw_url)
            # If the response was successful, no Exception will be raised
            response.raise_for_status()
        except requests.HTTPError as http_err:
            print(f"HTTP error occurred: {http_err}")
            return self
        except Exception as err:
            print(f"Other error occurred: {err}")

        self.readme = response.text
        return self

    @staticmethod
    def print_state(check_name="", state=None, indent=8):
        if state is True:
            print(" " * indent + check_name + ": " + Style.BRIGHT + Fore.GREEN + str(state).lower() + Style.RESET_ALL)
        elif state is False:
            print(" " * indent + check_name + ": " + str(state).lower())



from . import mixins
from .HowFairIsChecker import HowFairIsChecker


__author__ = "https://github.com/jspaaks"
__email__ = 'j.spaaks@esciencecenter.nl'

__all__ = [
    "HowFairIsChecker",
    "mixins"
]



import re
import requests


class CitationMixin:
    def has_zenodo_badge(self):
        if self.readme is None:
            self.print_state(check_name="has_zenodo_badge", state=False)
            return False
        regex = r"!\[.*\]\(https://zenodo\.org/badge/DOI/10\.5281/zenodo" + \
                r"\.[0-9]*\.svg\)\]\(https://doi\.org/10\.5281/zenodo\.[0-9]*\)"
        r = re.compile(regex).search(self.readme) is not None
        self.print_state(check_name="has_zenodo_badge", state=r)
        return r

    def has_citation_file(self):
        url = "https://raw.githubusercontent.com/" + \
              "{0}/{1}/{2}/CITATION".format(self.owner, self.repo, self.branch)
        try:
            response = requests.get(url)
            # If the response was successful, no Exception will be raised
            response.raise_for_status()
        except requests.HTTPError:
            self.print_state(check_name="has_citation_file", state=False)
            return False
        except Exception as err:
            print(f"Other error occurred: {err}")
        self.print_state(check_name="has_citation_file", state=True)
        return True

    def has_citationcff_file(self):
        url = "https://raw.githubusercontent.com/" + \
              "{0}/{1}/{2}/CITATION.cff".format(self.owner, self.repo, self.branch)
        try:
            response = requests.get(url)
            # If the response was successful, no Exception will be raised
            response.raise_for_status()
        except requests.HTTPError:
            self.print_state(check_name="has_citationcff_file", state=False)
            return False
        except Exception as err:
            print(f"Other error occurred: {err}")
        self.print_state(check_name="has_citationcff_file", state=True)
        return True

    def has_zenodo_metadata_file(self):
        url = "https://raw.githubusercontent.com/" + \
              "{0}/{1}/{2}/.zenodo.json".format(self.owner, self.repo, self.branch)
        try:
            response = requests.get(url)
            # If the response was successful, no Exception will be raised
            response.raise_for_status()
        except requests.HTTPError:
            self.print_state(check_name="has_zenodo_metadata_file", state=False)
            return False
        except Exception as err:
            print(f"Other error occurred: {err}")
        self.print_state(check_name="has_zenodo_metadata_file", state=True)
        return True

    def has_codemeta_file(self):
        url = "https://raw.githubusercontent.com/" + \
              "{0}/{1}/{2}/codemeta.json".format(self.owner, self.repo, self.branch)
        try:
            response = requests.get(url)
            # If the response was successful, no Exception will be raised
            response.raise_for_status()
        except requests.HTTPError:
            self.print_state(check_name="has_codemeta_file", state=False)
            return False
        except Exception as err:
            print(f"Other error occurred: {err}")
        self.print_state(check_name="has_codemeta_file", state=True)
        return True



import sys
from colorama import init as init_terminal_colors

from howfairis import HowFairIsChecker


def cli():

    init_terminal_colors()

    print("Checking compliance with fair-software.eu...")

    if len(sys.argv) != 2:
        raise Exception("Expected exactly one argument, i.e. the URL for " +
                        "which GitHub repository to run the analysis.")

    url = sys.argv[1]
    print("Running for {0}\n".format(url))
    checker = HowFairIsChecker(url)
    checker.deconstruct_url()
    checker.get_readme()
    checker.check_repository()
    checker.check_license()
    checker.check_registry()
    checker.check_citation()
    checker.check_checklist()
    checker.check_badge()


if __name__ == "__main__":
    cli()



from .RepositoryMixin import RepositoryMixin
from .LicenseMixin import LicenseMixin
from .RegistryMixin import RegistryMixin
from .CitationMixin import CitationMixin
from .ChecklistMixin import ChecklistMixin


__all__ = [
    "RepositoryMixin",
    "LicenseMixin",
    "RegistryMixin",
    "CitationMixin",
    "ChecklistMixin"
]



import re


class ChecklistMixin:
    def has_core_infrastructures_badge(self):
        if self.readme is None:
            self.print_state(check_name="has_core_infrastructures_badge", state=False)
            return False
        regex = r"!\[.*\]\(https://bestpractices\.coreinfrastructure\.org" + \
                r"/projects/[0-9]*/badge\)\]\(https://bestpractices\." + \
                r"coreinfrastructure\.org/projects/[0-9]*\)"
        r = re.compile(regex).search(self.readme) is not None
        self.print_state(check_name="has_core_infrastructures_badge", state=r)
        return r

    def has_sonarcloud_badge(self):
        if self.readme is None:
            self.print_state(check_name="has_sonarcloud_badge", state=False)
            return False
        regex = r"!\[.*\]\(https://sonarcloud\.io/api/project_badges/.*\)\]" + \
                r"\(https://sonarcloud\.io/dashboard\?id=.*\)"
        r = re.compile(regex).search(self.readme) is not None
        self.print_state(check_name="has_sonarcloud_badge", state=r)
        return r



import requests


class RepositoryMixin:

    def has_open_repository(self):
        url = "https://api.github.com/repos/{0}/{1}".format(self.owner, self.repo)

        try:
            response = requests.get(url)
            # If the response was successful, no Exception will be raised
            response.raise_for_status()
        except requests.HTTPError:
            self.print_state(check_name="has_open_repository", state=False)
            return False
        except Exception as err:
            print(f"Other error occurred: {err}")
        self.print_state(check_name="has_open_repository", state=True)
        return True



import re
import requests


class RegistryMixin:
    def has_pypi_badge(self):
        if self.readme is None:
            self.print_state(check_name="has_pypi_badge", state=False)
            return False
        regex = r"!\[.*\]\(https://img\.shields\.io/pypi/v/[^.]*\.svg.*\)\]" + \
                r"\(https://pypi\.python\.org/pypi/.*\)"
        r = re.compile(regex).search(self.readme) is not None
        self.print_state(check_name="has_pypi_badge", state=r)
        return r

    def has_bintray_badge(self):
        if self.readme is None:
            self.print_state(check_name="has_bintray_badge", state=False)
            return False
        regex = r"!\[.*\]\(https://api\.bintray\.com/packages" + \
                r"/.*/.*/.*/images/download\.svg\)\]" + \
                r"\(https://bintray\.com/.*/.*/.*/.*\)"
        r = re.compile(regex).search(self.readme) is not None
        self.print_state(check_name="has_bintray_badge", state=r)
        return r

    def has_conda_badge(self):
        if self.readme is None:
            self.print_state(check_name="has_conda_badge", state=False)
            return False
        regex = r"!\[.*\]\(https://anaconda\.org/.*/.*/badges" + \
                r"/installer/conda\.svg\)\]" + \
                r"\(https://anaconda\.org/.*/.*\)"
        r = re.compile(regex).search(self.readme) is not None
        self.print_state(check_name="has_conda_badge", state=r)
        return r

    def is_on_github_marketplace(self):
        try:
            response = requests.get(self.url)
            # If the response was successful, no Exception will be raised
            response.raise_for_status()
        except requests.HTTPError as http_err:
            print(f"HTTP error occurred: {http_err}")
            self.print_state(check_name="is_on_github_marketplace", state=False)
            return False
        except Exception as err:
            print(f"Other error occurred: {err}")

        html = response.text
        r = "Use this GitHub Action with your project" in html and \
            "Add this Action to an existing workflow or create a new one." in html
        self.print_state(check_name="is_on_github_marketplace", state=r)
        return r



import os

from setuptools import setup


here = os.path.abspath(os.path.dirname(__file__))

with open("README.rst") as readme_file:
    readme = readme_file.read()

setup(
    name="howfairis",
    entry_points={
        "console_scripts": ["howfairis=howfairis.cli:cli"],
    },
    version="0.4.0",
    description="Python package to analyze compliance with fair-software.eu recommendations",
    long_description=readme + "\n\n",
    author="https://github.com/jspaaks",
    author_email="j.spaaks@esciencecenter.nl",
    url="https://github.com/fair-software/howfairis",
    packages=["howfairis", "howfairis.mixins"],
    include_package_data=True,
    license="Apache Software License 2.0",
    zip_safe=False,
    keywords="howfairis",
    classifiers=[
        "Development Status :: 2 - Pre-Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: Apache Software License",
        "Natural Language :: English",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
    ],
    test_suite="tests",
    install_requires=[
        "requests",
        "colorama"
    ],
    setup_requires=[
    ],
    tests_require=[
    ],
    extras_require={
        "dev":  [
            "prospector[with_pyroma]",
            "yapf",
            "bumpversion",
            "pytest",
            "pytest-cov",
            "pycodestyle",
            "pytest-runner",
            "twine"
        ],
    },
    data_files=[]
)
